/* headerview.c */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>
#include <gtk/gtkwidget.h>
#include <gtk/gtkstyle.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtktext.h>
#include <gtk/gtkpixmap.h>
#include <gtk/gtkcontainer.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#if HAVE_LIBCOMPFACE
#  include <compface.h>
#endif

#include "intl.h"
#include "main.h"
#include "headerview.h"
#include "prefs_common.h"
#include "gtkutils.h"
#include "utils.h"

static GdkFont *normalfont;
static GdkFont *boldfont;

#if HAVE_LIBCOMPFACE
#define XPM_XFACE_HEIGHT	(HEIGHT + 3)  /* 3 = 1 header + 2 colors */

static gchar *xpm_xface[XPM_XFACE_HEIGHT];

static void headerview_show_xface	(HeaderView	*headerview,
					 MsgInfo	*msginfo);
static gint create_xpm_from_xface	(gchar		*xpm[],
					 const gchar	*xface);
#endif

HeaderView *headerview_create(void)
{
	HeaderView *headerview;
	GtkWidget *hbox;
	GtkWidget *text;

	debug_print(_("Creating header view...\n"));
	headerview = g_new0(HeaderView, 1);

	hbox = gtk_hbox_new(FALSE, 0);

	text = gtk_text_new(NULL, NULL);
	GTK_WIDGET_UNSET_FLAGS(text, GTK_CAN_FOCUS);
	gtk_text_set_word_wrap(GTK_TEXT(text), FALSE);
	gtk_text_set_line_wrap(GTK_TEXT(text), FALSE);

	gtk_box_pack_start(GTK_BOX(hbox), text, TRUE, TRUE, 0);

	headerview->hbox = hbox;
	headerview->text = text;
	headerview->image = NULL;

	gtk_widget_show_all(hbox);

	return headerview;
}

void headerview_init(HeaderView *headerview)
{
	GtkStyle *style;
	gint height;

	//if (!normalfont)
	//	normalfont = gdk_fontset_load(NORMAL_FONT);
	if (!boldfont)
		boldfont = gdk_fontset_load(BOLD_FONT);

	style = gtk_style_copy(gtk_widget_get_style(headerview->text));
	//style = gtk_widget_get_style(headerview->text);
	style->base[GTK_STATE_NORMAL] = style->bg[GTK_STATE_NORMAL];
	//style->font = normalfont;
	gtk_widget_set_style(headerview->text, style);

	height = gtkut_get_font_height(style->font);
	gtk_widget_set_usize(headerview->text, -1, height * 2 + 10);

#if HAVE_LIBCOMPFACE
	{
		gint i;

		for (i = 0; i < XPM_XFACE_HEIGHT; i++) {
			xpm_xface[i] = g_malloc(WIDTH + 1);
			*xpm_xface[i] = '\0';
		}
	}
#endif

#if 0
	_("From: ");
	_("To: ");
	_("Newsgroups: ");
	_("Subject: ");
#endif
}

#define TR(str)	(prefs_common.trans_hdr ? gettext(str) : str)

void headerview_show(HeaderView *headerview, MsgInfo *msginfo)
{
	GtkText *text = GTK_TEXT(headerview->text);

	headerview_clear(headerview);
	gtk_text_freeze(text);

	gtk_text_insert(text, boldfont  , NULL, NULL, TR("From: "), -1);
	gtk_text_insert(text, normalfont, NULL, NULL,
			msginfo->from ? msginfo->from : _("(No From)"), -1);

	if (msginfo->to) {
		gtk_text_insert(text, NULL      , NULL, NULL, "  " , 2);
		gtk_text_insert(text, boldfont  , NULL, NULL, TR("To: ") , -1);
		gtk_text_insert(text, normalfont, NULL, NULL, msginfo->to , -1);
	}
	if (msginfo->newsgroups) {
		gtk_text_insert(text, NULL, NULL, NULL, "  ", 2);
		gtk_text_insert(text, boldfont, NULL, NULL, TR("Newsgroups: "), -1);
		gtk_text_insert(text, normalfont, NULL, NULL,
				msginfo->newsgroups, -1);
	}
	gtk_text_insert(text, NULL      , NULL, NULL, "\n", 1);

	gtk_text_insert(text, boldfont  , NULL, NULL, TR("Subject: "), -1);
	gtk_text_insert(text, normalfont, NULL, NULL,
			msginfo->subject ? msginfo->subject :
			_("(No Subject)"), -1);
	gtk_text_thaw(text);

#if HAVE_LIBCOMPFACE
	headerview_show_xface(headerview, msginfo);
#endif
}

#if HAVE_LIBCOMPFACE
static void headerview_show_xface(HeaderView *headerview, MsgInfo *msginfo)
{
	gchar xface[2048];
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GtkWidget *hbox = headerview->hbox;

	if (!msginfo->xface) {
		if (headerview->image &&
		    GTK_WIDGET_VISIBLE(headerview->image)) {
			gtk_widget_hide(headerview->image);
			gtk_widget_queue_resize(hbox);
		}
		return;
	}

	strncpy(xface, msginfo->xface, sizeof(xface));

	if (uncompface(xface) < 0) {
		g_warning("uncompface failed\n");
		if (headerview->image)
			gtk_widget_hide(headerview->image);
		return;
	}

	create_xpm_from_xface(xpm_xface, xface);

	pixmap = gdk_pixmap_create_from_xpm_d
		(hbox->window, &mask, &hbox->style->white, xpm_xface);

	if (!headerview->image) {
		GtkWidget *image;

		image = gtk_pixmap_new(pixmap, mask);
		gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
		gtk_widget_show(image);
		headerview->image = image;
	} else {
		gtk_pixmap_set(GTK_PIXMAP(headerview->image), pixmap, mask);
		gtk_widget_show(headerview->image);
	}

	gdk_pixmap_unref(pixmap);
}
#endif

void headerview_clear(HeaderView *headerview)
{
	GtkText *text = GTK_TEXT(headerview->text);

	gtk_text_freeze(text);
	gtk_text_backward_delete(text, gtk_text_get_length(text));
	gtk_text_thaw(text);

	if (headerview->image && GTK_WIDGET_VISIBLE(headerview->image)) {
		gtk_widget_hide(headerview->image);
		gtk_widget_queue_resize(headerview->hbox);
	}
}

#if HAVE_LIBCOMPFACE
static gint create_xpm_from_xface(gchar *xpm[], const gchar *xface)
{
	static gchar *bit_pattern[] = {
		"....",
		"...#",
		"..#.",
		"..##",
		".#..",
		".#.#",
		".##.",
		".###",
		"#...",
		"#..#",
		"#.#.",
		"#.##",
		"##..",
		"##.#",
		"###.",
		"####"
	};

	static gchar *xface_header = "48 48 2 1";
	static gchar *xface_black  = "# c #000000";
	static gchar *xface_white  = ". c #ffffff";

	gint i, line = 0;
	const guchar *p;
	gchar buf[WIDTH * 4 + 1];  /* 4 = strlen("0x0000") */

	p = xface;

	strcpy(xpm[line++], xface_header);
	strcpy(xpm[line++], xface_black);
	strcpy(xpm[line++], xface_white);

	for (i = 0; i < HEIGHT; i++) {
		gint col;

		buf[0] = '\0';
     
		for (col = 0; col < 3; col++) {
			gint figure;

			p += 2;  /* skip '0x' */

			for (figure = 0; figure < 4; figure++) {
				gint n = 0;

				if ('0' <= *p && *p <= '9') {
					n = *p - '0';
				} else if ('a' <= *p && *p <= 'f') {
					n = *p - 'a' + 10;
				} else if ('A' <= *p && *p <= 'F') {
					n = *p - 'A' + 10;
				}

				strcat(buf, bit_pattern[n]);
				p++;  /* skip ',' */
			}

			p++;  /* skip '\n' */
		}

		strcpy(xpm[line++], buf);
		p++;
	}

	return 0;
}
#endif
